/*
	$Revision: 1.1 $ $Date: 2006/01/14 22:26:55 $

	S interface

	This business is necessary because the programs use
	internally-defined data structures (see func.h, geom.h)
	and we need to convert to & from S objects.

*/

#include "geom.h"
#include "func.h"

char *malloc();

/*
	ALLOCATION OF STRUCTURES/ARRAYS

*/

Ftable *
allocFtable(n)		/* allocate function table of size n */
int	n;
{
	Ftable *x;
	x = (Ftable *) malloc(sizeof(Ftable));
	x->n = n;
	x->f 	 = (double *) malloc((unsigned ) (n * sizeof(double)));
	x->num 	 = (double *) malloc((unsigned ) (n * sizeof(double)));
	x->denom = (double *) malloc((unsigned ) (n * sizeof(double)));
	return(x);
}

freeFtable(x) Ftable *x; 
{ free((char *) x->f); free((char *) x->num); free((char *) x->denom); }

Itable	*
allocItable(n)
int	n;
{
	Itable	*x;
	x = (Itable *) malloc(sizeof(Itable));
	x->n = n;
	x->num 	 = (int *) malloc((unsigned ) (n * sizeof(int)));
	x->denom = (int *) malloc((unsigned ) (n * sizeof(int)));
	return(x);
}

freeItable(x) Itable *x; 
{ free((char *) x->num); free((char *) x->denom); }

Box	*
allocBox()		/* I know this is ridiculous but it's consistent. */
{
	return((Box *) malloc(sizeof(Box)));
}

freeBox(x) Box *x; { }


Point	*
allocParray(n)		/* allocate array of n Points */
int	n;
{
	return((Point *) malloc((unsigned ) n * sizeof(Point)));
}

freeParray(x) Point *x; { free((char *) x); }

double	*
allocDarray(n)		/* allocate array of n doubles */
int	n;
{
	return((double *) malloc((unsigned ) n * sizeof(double)));
}

freeDarray(x) double *x; { free((char *) x); }

/*
	CONVERSION OF DATA TYPES 

		S -> internal

	including allocation of internal data types as needed
*/

Point	*
S2Pointarray(x,y,z,n)
	double *x, *y, *z;
	long	*n;
{
	int	i, nn;
	Point	*p;

	nn = *n;
	p = allocParray(nn);
	
	for(i = 0; i < nn; i++)
	{
		p[i].x = x[i];
		p[i].y = y[i];
		p[i].z = z[i];
	}
	return(p);
}

Box *
S2Box(x0, x1, y0, y1, z0, z1)
	double	*x0, *x1, *y0, *y1, *z0, *z1;
{
	Box *b;
	b = allocBox();

	b->x0 = *x0;
	b->x1 = *x1;
	b->y0 = *y0;
	b->y1 = *y1;
	b->z0 = *z0;
	b->z1 = *z1;
	return(b);
}

Ftable *
S2Ftable(t0, t1, n, f, num, denom)
	double *t0, *t1;
	long	*n;
	double	*f, *num, *denom;
{
	Ftable	*tab;
	int	i, nn;

	nn = *n;
	tab = allocFtable(nn);

	tab->t0 = *t0;
	tab->t1 = *t1;

	for(i = 0; i < nn; i++)
	{
		tab->f[i] = f[i];
		tab->num[i] = num[i];
		tab->denom[i] = denom[i];
	}
	return(tab);
}
	
Itable	*
S2Itable(t0, t1, n, num, denom)
	double	*t0, *t1;
	long *n;
	long	*num, *denom;
{
	Itable	*tab;
	int	i, nn;

	nn = *n;
	tab = allocItable(nn);

	tab->t0 = *t0;
	tab->t1 = *t1;

	for(i = 0; i < nn; i++)
	{
		tab->num[i] = num[i];
		tab->denom[i] = denom[i];
	}
	return(tab);
}

/*
	CONVERSION OF DATA TYPES 

		internal -> S

	Note: it can generally be assumed that the S arguments
	are already allocated vectors of correct length,
	so we do not allocate them.

	We free up the space in the internal arrays.

*/

Ftable2S(tab, t0, t1, n, f, num, denom)
		/* internal */
	Ftable	*tab;
		/* S representation */
	double	*t0, *t1;
	long	*n;
	double	*f, *num, *denom;
{
	int	i;

	*t0 = tab->t0;
	*t1 = tab->t1;
	*n = tab->n;
	
	for(i = 0; i < tab->n; i++)
	{
		f[i] = tab->f[i];
		num[i] = tab->num[i];
		denom[i] = tab->denom[i];
	}

	freeFtable(tab);
}

Itable2S(tab, t0, t1, m, num, denom)
		/* internal */
	Itable	*tab;
		/* S representation */
	double	*t0, *t1;
	long  *m;
	long	*num, *denom;
{
	int	i;

	*t0 = tab->t0;
	*t1 = tab->t1;
	*m  = tab->n;

	for(i = 0; i < tab->n; i++)
	{
		num[i] = tab->num[i];
		denom[i] = tab->denom[i];
	}

	freeItable(tab);
}
	
		
/*
	S CALLING INTERFACE 

	These routines are called from S by 
	> .C("routine-name", ....)
*/

void
Scall_k3(x,y,z, n, x0, x1, y0, y1, z0, z1, t0, t1, m, f, num, denom, method)

	double	*x, *y, *z;	/* points */
	long	*n;

	double	*x0, *x1, 	/* box */
		*y0, *y1, 
		*z0, *z1;	

	double	*t0, *t1;	/* Ftable */
	long	*m;
	double	*f, *num, *denom;

	long	*method;
	
{
	Point	*p;
	Box 	*b;
	Ftable	*tab;
	
	p = S2Pointarray(x, y, z, n);
	b = S2Box(x0, x1, y0, y1, z0, z1);
	tab = S2Ftable(t0, t1, m, f, num, denom);	

	switch((int) *method) {	
	case 0:
		k3trans(p, (int) *n, b, tab); break;
	case 1:
		k3isot(p, (int) *n, b, tab); break;
	default:
	  printf("No such method %d\n", *method);
	}
		
	Ftable2S(tab, t0, t1, m, f, num, denom);
}

void
Scall_g3(x,y,z, n, x0, x1, y0, y1, z0, z1, t0, t1, m, f, num, denom, method)

	double	*x, *y, *z;	/* points */
	long	*n;

	double	*x0, *x1, 	/* box */
		*y0, *y1, 
		*z0, *z1;	

	double	*t0, *t1;	/* Ftable */
	long	*m;
	double	*f, *num, *denom;

	long	*method;
{
	Point	*p;
	Box 	*b;
	Ftable	*tab;
	
	p = S2Pointarray(x, y, z, n);
	b = S2Box(x0, x1, y0, y1, z0, z1);
	tab = S2Ftable(t0, t1, m, f, num, denom);	

	switch(*method) {
	case 1:
		g3one(p, (int) *n, b, tab); 
		break;
	case 3:
		g3three(p, (int) *n, b, tab); 
		break;
	default:
		printf("Method %d not implemented: defaults to 3\n", *method);
		g3three(p, (int) *n, b, tab); 
	}
		
	Ftable2S(tab, t0, t1, m, f, num, denom);
}

void
Scall_f3(x,y,z, n, x0, x1, y0, y1, z0, z1, 
	vside, 
	t0, t1, m, num, denom, method)

	double	*x, *y, *z;	/* points */
	long	*n;

	double	*x0, *x1, 	/* box */
		*y0, *y1, 
		*z0, *z1;	

	double	*vside;

	double 	*t0, *t1;
	long     *m;		/* Itable */
	long	*num, *denom;

	long	*method;
{
	Point	*p;
	Box 	*b;
	Itable	*count;
	
	printf("Inside Scall_f3\n");
	p = S2Pointarray(x, y, z, n);
	b = S2Box(x0, x1, y0, y1, z0, z1);
	count = S2Itable(t0, t1, m, num, denom);	

	switch((int) *method) {
	case 0:
		phatnaive(p, (int) *n, b, *vside, count);
		break;
	case 1:
		phatminus(p, (int) *n, b, *vside, count);
	}

	Itable2S(count, t0, t1, m, num, denom);
}
