/* tetraarea.c --- c code for calculating the area of a tetrahedron */

 /* Dan Lofgreen 2 April 2007
 *
 * This software is distributed free under the conditions that
 * (1) it shall not be incorporated in software that is subsequently sold
 * (2) the authorship of the software shall be acknowledged in any 
 *     publication that uses results generated by the software
 * (3) this notice shall remain in place in each file.
 *
 * GPL applies: http://www.gnu.org/copyleft/gpl.html
 */ 
 
/*
 * See SpatialAnalysis.h file for declaration details
 *
 */


#include <math.h>

/* tetraarea calculates the area of a tetrahedron in 3D space */
void tetraarea(double *x, double *y, double *z, double *si, double *Ai)
{
    int i;
    double s;
        
    /* Calculate the lengths of the six line segments making up
     * the tetrahedron */   
    si[0] = sqrt((x[1]-x[0])*(x[1]-x[0]) + (y[1]-y[0])*(y[1]-y[0])
        + (z[1]-z[0])*(z[1]-z[0]));
    si[1] = sqrt((x[2]-x[1])*(x[2]-x[1]) + (y[2]-y[1])*(y[2]-y[1])
        + (z[2]-z[1])*(z[2]-z[1]));
    si[2] = sqrt((x[0]-x[2])*(x[0]-x[2]) + (y[0]-y[2])*(y[0]-y[2])
        + (z[0]-z[2])*(z[0]-z[2]));
    si[3] = sqrt((x[3]-x[0])*(x[3]-x[0]) + (y[3]-y[0])*(y[3]-y[0])
        + (z[3]-z[0])*(z[3]-z[0]));
    si[4] = sqrt((x[3]-x[1])*(x[3]-x[1]) + (y[3]-y[1])*(y[3]-y[1])
        + (z[3]-z[1])*(z[3]-z[1]));
    si[5] = sqrt((x[3]-x[2])*(x[3]-x[2]) + (y[3]-y[2])*(y[3]-y[2])
        + (z[3]-z[2])*(z[3]-z[2]));
    
    /* Calculate the area for side #1 */
    s = (si[0] + si[1] + si[2])/2.0;
    
    /* Calculate the area as */
    Ai[0] = sqrt(s*(s-si[0])*(s-si[1])*(s-si[2]));
    
    /* Calculate the area for side #2 */
    s = (si[3] + si[0] + si[4])/2.0;
    
    /* Calculate the area as */
    Ai[1] = sqrt(s*(s-si[3])*(s-si[0])*(s-si[4]));
    
    /* Calculate the area for side #3 */
    s = (si[4] + si[1] + si[5])/2.0;
    
    /* Calculate the area as */
    Ai[2] = sqrt(s*(s-si[4])*(s-si[1])*(s-si[5]));
    
    /* Calculate the area for side #4 */
    s = (si[3] + si[2] + si[5])/2.0;
    
    /* Calculate the area as */
    Ai[3] = sqrt(s*(s-si[3])*(s-si[2])*(s-si[5]));
    
}
