function res = f3d(pts, box, range, vsize, method)
%F3D  Compute F function in 3d.
% RES = F3D(PTS,BOX,RANGE,VSIZE,METHOD) Invokes the mex function kfg_funs
% to perform the F-function calculation with the following arguments:
%
% PTS     array of size [N,3] where N is number of points.
% BOX     vector of length 6 giving [XLO XHI YLO YHI ZLO ZHI]
% RANGE   vector of length 3 [LO HI NVAL] saying the distances
%         at which F should be evaluated.
% VSIZE   size of a voxel (See notes).
% METHOD  integer: 
%         0 (no correction)
%         1 (minus sampling).
%
% Return values:
% matrix RES with 2 columns and N rows, giving the distance r, and
% the value of the F function at that distance r.
%
% The F,G,K functions need compiled only once together.  To compile the mex
% function kfg_funs under windows use the following:
%
%   mex -v -g  kfg_funs.c sint.c f3.c g3.c k3.c sphefrac.c sphevol.c
%
% See also: G3D, K3D

% First perform some error checking
if (nargin ~= 5)  
  error('Not enough input arguments')
end
res = kfg_funs(3, pts, box, range, vsize, method);
