function res = g3d(pts, box, range, method)
%G3D  Compute G function in 3d.
% RES = G3D(PTS,BOX,RANGE,METHOD) Invokes the mex function kfg_funs
% to perform the G-function calculation with the following arguments:
%
% PTS     array of size [N,3] where N is number of points.
% BOX     vector of length 6 giving [XLO XHI YLO YHI ZLO ZHI]
% RANGE   vector of length 3 [LO HI NVAL] saying the distances
%         at which G should be evaluated.
% METHOD  integer: 
%         1 (minus sampling).
%         3 (Hanisch G3).
%
% Return values:
% matrix RES with 2 columns and N rows, giving the distance r, and
% the value of the G function at that distance r.
%
% The F,G,K functions need compiled only once together.  To compile the mex
% function kfg_funs under windows use the following:
%
%   mex -v -g  kfg_funs.c sint.c f3.c g3.c k3.c sphefrac.c sphevol.c
%
% See also: F3D, K3D

% First perform some error checking
if (nargin ~= 4)
    error('Not enough input arguments')
end
res = kfg_funs(2, pts, box, range, method);
