function res = k3d(pts, box, range, method)
%K3D  Compute K function in 3d.
% RES = K3D(PTS,BOX,RANGE,METHOD) Invokes the mex function kfg_funs
% to perform the K-function calculation with the following arguments:
%
% PTS     array of size [N,3] where N is number of points.
% BOX     vector of length 6 giving [XLO XHI YLO YHI ZLO ZHI]
% RANGE   vector of length 3 [LO HI NVAL] saying the distances
%         at which K should be evaluated.
% METHOD  integer: 
%         0 (translation)
%         1 (isotropic)
%
% Return values:
% matrix RES with 2 columns and NVAL rows, giving the distance r, and
% the value of the K function at that distance r.
%
% The F,G,K functions need compiled only once together.  To compile the mex
% function kfg_funs under windows use the following:
%
%   mex -v -g  kfg_funs.c sint.c f3.c g3.c k3.c sphefrac.c sphevol.c
%
% See also: F3D, G3D

if (nargin ~= 4)
    error('Not enough input arguments')
end
res = kfg_funs(1, pts, box, range, method);
